@@ -2086,6 +2086,7 @@ const (
 	PodDisruptionBudgetKind   = "PodDisruptionBudget"
 	PriorityClassKind         = "PriorityClass"
 	VerticalPodAutoscalerKind = "VerticalPodAutoscaler"
+	ConsoleYAMLSampleKind     = "ConsoleYAMLSample"
 )
 
 var supportedKinds = map[string]struct{}{
@@ -2094,6 +2095,7 @@ var supportedKinds = map[string]struct{}{
 	PodDisruptionBudgetKind:   {},
 	PriorityClassKind:         {},
 	VerticalPodAutoscalerKind: {},
+	ConsoleYAMLSampleKind:     {},
 }
 
 // isSupported returns true if OLM supports this type of CustomResource.
