@@ -2,6 +2,8 @@
 package reconciler
 
 import (
+	"strings"
+
 	"github.com/operator-framework/api/pkg/operators/v1alpha1"
 	controllerclient "github.com/operator-framework/operator-lifecycle-manager/pkg/lib/controller-runtime/client"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/lib/operatorclient"
@@ -89,13 +91,15 @@ func NewRegistryReconcilerFactory(lister operatorlister.OperatorLister, opClient
 	}
 }
 
-func Pod(source *v1alpha1.CatalogSource, name string, image string, labels map[string]string, readinessDelay int32, livenessDelay int32) *v1.Pod {
-	// ensure catalog image is pulled always if catalog polling is configured
+func Pod(source *v1alpha1.CatalogSource, name string, image string, saName string, labels map[string]string, readinessDelay int32, livenessDelay int32) *v1.Pod {
+	// Ensure the catalog image is always pulled if the image is not based on a digest, measured by whether an "@" is included.
+	// See https://github.com/docker/distribution/blob/master/reference/reference.go for more info.
+	// This means recreating non-digest based catalog pods will result in the latest version of the catalog content being delivered on-cluster.
 	var pullPolicy v1.PullPolicy
-	if source.Spec.UpdateStrategy != nil {
-		pullPolicy = v1.PullAlways
-	} else {
+	if strings.Contains(image, "@") {
 		pullPolicy = v1.PullIfNotPresent
+	} else {
+		pullPolicy = v1.PullAlways
 	}
 
 	pod := &v1.Pod{
@@ -118,7 +122,7 @@ func Pod(source *v1alpha1.CatalogSource, name string, image string, labels map[s
 					ReadinessProbe: &v1.Probe{
 						Handler: v1.Handler{
 							Exec: &v1.ExecAction{
-								Command: []string{"grpc_health_probe", "-addr=localhost:50051"},
+								Command: []string{"grpc_health_probe", "-addr=:50051"},
 							},
 						},
 						InitialDelaySeconds: readinessDelay,
@@ -127,7 +131,7 @@ func Pod(source *v1alpha1.CatalogSource, name string, image string, labels map[s
 					LivenessProbe: &v1.Probe{
 						Handler: v1.Handler{
 							Exec: &v1.ExecAction{
-								Command: []string{"grpc_health_probe", "-addr=localhost:50051"},
+								Command: []string{"grpc_health_probe", "-addr=:50051"},
 							},
 						},
 						InitialDelaySeconds: livenessDelay,
@@ -142,8 +146,9 @@ func Pod(source *v1alpha1.CatalogSource, name string, image string, labels map[s
 				},
 			},
 			NodeSelector: map[string]string{
-				"beta.kubernetes.io/os": "linux",
+				"kubernetes.io/os": "linux",
 			},
+			ServiceAccountName: saName,
 		},
 	}
 	return pod
