@@ -1,14 +1,15 @@
 package client
 
 import (
-	"github.com/operator-framework/operator-lifecycle-manager/pkg/package-server/client/clientset/versioned"
 	"k8s.io/client-go/rest"
+
+	"github.com/operator-framework/operator-lifecycle-manager/pkg/package-server/client/clientset/versioned"
 )
 
 // NewClient creates a client that can interact with the ALM resources in k8s api
 func NewClient(kubeconfig string) (client versioned.Interface, err error) {
 	var config *rest.Config
-	config, err = getConfig(kubeconfig)
+	config, err = GetConfig(kubeconfig)
 	if err != nil {
 		return
 	}
